from math import pi, sqrt, log
import numpy as np
from scipy.interpolate import InterpolatedUnivariateSpline
from scipy.optimize import fsolve, brentq, fmin  

 
# Define the parameters.
J = 10000
T = 480
beta = .9966
muh = 1.
mul = 0.
 
def likelihood(mu, x, t, sigma):
    y = ((x-t*mu)/(sigma*sqrt(2.*t)))
    return np.exp(-y*y)
  
def f(x, p, t, sigma):
    a = p*likelihood(muh, x, t, sigma)
    return a/(a+(1-p)*likelihood(mul, x, t, sigma)) 
        
def w(p):
    return p*muh + (1-p)*mul
     
# Define the points and weights to be used.
x1 = np.array((2.45340708300901249903*10**-1,7.37473728545394358719*10**-1, 
    1.23407621539532300786, 1.73853771211658620678,
    2.25497400208927552311, 2.78880605842813048055,
    3.34785456738321632688, 3.94476404011562521040, 
    4.60368244955074427298, 5.38748089001123286199))
  
points = np.sort(np.concatenate((-x1,x1)))
  
w1 = np.array((4.62243669600610089640*10**-1, 2.86675505362834129720*10**-1, 
    1.09017206020023320014*pow(10,-1), 2.48105208874636108814*pow(10,-2), 
    3.24377334223786183217*pow(10,-3), 2.28338636016353967260*pow(10,-4), 
    7.80255647853206369398*pow(10,-6), 1.08606937076928169398*pow(10,-7), 
    4.39934099227318055366*pow(10,-10), 2.22939364553415129254*pow(10,-13)))
  
weights = np.concatenate((w1[::-1],w1))        
 
def get_sigma(params):
    p0, delta, alpha, s, lamb = params
     
    def randwage(sig, n):
        if np.random.rand() < p0:
            return f(sqrt(12)*sig*np.random.randn(n)+12*muh, p0, 12, sig)
        else:
            return f(sqrt(12)*sig*np.random.randn(n)+12*mul, p0, 12, sig)
  
    def wage_sd(sig):
        log_wages = np.log(randwage(sig, 100000)/p0)
        if (log_wages>0).sum()>50000:
            return wage_sd(sig)
        return np.std(log_wages)
  
    def get_sigma(sig):
        return wage_sd(sig) - .3555
  
    sigma = brentq(get_sigma, 0.2, 8.)
    return np.round(sigma,2)
     
def get_VfunT(params,sigma):
    p0, delta, alpha, s, lamb = params
    b = .5*p0
     
    n0 = 101
    m = len(points)
  
    xgrid = np.linspace(0,1,n0)
    Vgrid0 = np.linspace(282, 285, n0)
    Vfun = InterpolatedUnivariateSpline(xgrid, Vgrid0)
  
    Un0 = 0.
    UnT = 200.
  
    diff = 1.
    j = 0
  
    Un_denom = (1-beta*(1-lamb))
    Un_coeff = beta*lamb/sqrt(pi)
  
    points1 = sqrt(2*alpha*T)*sigma*points+alpha*T*muh
    points2 = sqrt(2*alpha*T)*sigma*points+alpha*T*mul
    points3 = sqrt(2)*sigma*points+muh
    points4 = sqrt(2)*sigma*points+mul
    f1 = f(points1, p0, alpha*T, sigma)
    f2 = f(points2, p0, alpha*T, sigma)
    f3 = f(points3, np.vstack(xgrid), 1, sigma)
    f4 = f(points4, np.vstack(xgrid), 1, sigma)
  
    while diff > 1e-3:
  
        Vfun1 = Vfun(f1)
        Vfun2 = Vfun(f2)
  
        def Integral1(U):
            mask = Vfun1 < U
            Vfun1[mask] = U
            return weights.dot(Vfun1)
  
        def Integral2(U):
            mask = Vfun2 < U
            Vfun2[mask] = U
            return weights.dot(Vfun2)
  
        def get_U(Un):
            return Un - (b+Un_coeff*(p0*Integral1(Un)
                                     +(1-p0)*Integral2(Un)))/Un_denom
  
        UnT = fsolve(get_U, UnT)
  
        Vfun3 = np.array(map(Vfun, f3))
  
        Integrand3a = Vfun1*np.ones((m, n0, m))
        mask = (Integrand3a.T < Vfun3).T
        Integrand3a[mask] = (mask*Vfun3)[mask]
  
        Integrand3b = Vfun2*np.ones((m, n0, m))
        mask = (Integrand3b.T < Vfun3).T
        Integrand3b[mask] = (mask*Vfun3)[mask]    
  
        Integral3a = Integrand3a.dot(weights).T
        Integral3b = Integrand3b.dot(weights).T
  
        Integral3 = ((1-delta)*Vfun3
                       +delta*(p0*Integral3a+(1-p0)*Integral3b)/sqrt(pi))
        mask = Integral3 < UnT
        Integral3[mask] = UnT
        Integral3 = Integral3.dot(weights)
  
        Vfun4 = np.array(map(Vfun, f4))
  
        Integrand4a = Integrand3a
        mask = (Integrand4a.T < Vfun4).T
        Integrand4a[mask] = (mask*Vfun4)[mask]
  
        Integrand4b = Integrand3b
        mask = (Integrand4b.T < Vfun4).T
        Integrand4b[mask] = (mask*Vfun4)[mask]    
  
        Integral4a = Integrand4a.dot(weights).T
        Integral4b = Integrand4b.dot(weights).T
  
        Integral4 = ((1-delta)*Vfun4
                       +delta*(p0*Integral4a+(1-p0)*Integral4b)/sqrt(pi))
        mask = Integral4 < UnT
        Integral4[mask] = UnT
        Integral4 = Integral4.dot(weights)
  
        Vgrid = w(xgrid)+beta*(s*UnT+(1-s)*((xgrid*Integral3
                                           +(1-xgrid)*Integral4))/sqrt(pi))
  
        Vfun = InterpolatedUnivariateSpline(xgrid, Vgrid)
  
        Vdiff = np.max(np.abs(Vgrid0-Vgrid))
        Udiff = np.abs(UnT-Un0)
        diff = max(Vdiff, Udiff)
  
        j += 1
  
        Vgrid0 = Vgrid
        Un0 = UnT
         
    VfunT = Vfun
    return VfunT, UnT
     
     
def get_bounds(params,sigma):
    p0, delta, alpha, s, lamb = params
    b = .5*p0
     # Solve without truncation and return bounds
    VfunT, UnT = get_VfunT(params,sigma)
     
    Vfun_list = []
    Un_list = []
    Bounds = np.zeros(T+1)
    Bound = 1.
    t = T
     
    n0 = 101
    m = len(points)
    xgrid = np.linspace(0,1,n0)
    points3 = sqrt(2)*sigma*points+muh
    points4 = sqrt(2)*sigma*points+mul
    f3 = f(points3, np.vstack(xgrid), 1, sigma)
    f4 = f(points4, np.vstack(xgrid), 1, sigma)
     
    Un_denom = (1-beta*(1-lamb))
    Un_coeff = beta*lamb/sqrt(pi)
     
    while t > 0:
        points1 = sqrt(2*alpha*t)*sigma*points+alpha*t*muh
        points2 = sqrt(2*alpha*t)*sigma*points+alpha*t*mul
  
        if t == T:
            Vfun = VfunT
            Un = UnT
        else:
            Vfun = Vfun_list[-1]
            Un = Un_list[-1]
             
        # Make the Vgrid.
        Vfun1 = Vfun(f(points1, p0, alpha*t, sigma))
        Vfun2 = Vfun(f(points2, p0, alpha*t, sigma))
        Vfun3 = np.array(map(Vfun, f3))
  
        Integrand3a = Vfun1*np.ones((m, n0, m))
        mask = (Integrand3a.T < Vfun3).T
        Integrand3a[mask] = (mask*Vfun3)[mask]
  
        Integrand3b = Vfun2*np.ones((m, n0, m))
        mask = (Integrand3b.T < Vfun3).T
        Integrand3b[mask] = (mask*Vfun3)[mask]    
  
        Integral3a = Integrand3a.dot(weights).T
        Integral3b = Integrand3b.dot(weights).T
  
        Integral3 = ((1-delta)*Vfun3
                       +delta*(p0*Integral3a+(1-p0)*Integral3b)/sqrt(pi))
        mask = Integral3 < Un
        Integral3[mask] = Un
        Integral3 = Integral3.dot(weights)
  
        Vfun4 = np.array(map(Vfun, f4))
  
        Integrand4a = Integrand3a
        mask = (Integrand4a.T < Vfun4).T
        Integrand4a[mask] = (mask*Vfun4)[mask]
  
        Integrand4b = Integrand3b
        mask = (Integrand4b.T < Vfun4).T
        Integrand4b[mask] = (mask*Vfun4)[mask]    
  
        Integral4a = Integrand4a.dot(weights).T
        Integral4b = Integrand4b.dot(weights).T
  
        Integral4 = ((1-delta)*Vfun4
                       +delta*(p0*Integral4a+(1-p0)*Integral4b)/sqrt(pi))
        mask = Integral4 < Un
        Integral4[mask] = Un
        Integral4 = Integral4.dot(weights)
  
        Vgrid = w(xgrid)+beta*(s*Un+(1-s)*((xgrid*Integral3
                                           +(1-xgrid)*Integral4))/sqrt(pi))
  
        Vfun = InterpolatedUnivariateSpline(xgrid, Vgrid)
        Vfun_list.append(Vfun)
  
        # Get Un(t)    
        def Integral1(U):
            V = Vfun(f(points1, p0, alpha*t, sigma))
            mask = V < U
            V[mask] = U
            return weights.dot(V)
  
        def Integral2(U):
            V = Vfun(f(points2, p0, alpha*t, sigma))
            mask = V < U
            V[mask] = U
            return weights.dot(V)
  
        def get_U(Un):
            return Un - (b+Un_coeff*(p0*Integral1(Un)
                                     +(1-p0)*Integral2(Un)))/Un_denom
  
        Un = fsolve(get_U, Un)
        Un_list.append(Un)  
  
        # Get bound.
        def get_bound(x):
            return delta*max(Vfun(x), Vfun(p0))+(1-delta)*Vfun(x)-Un
  
        Bound = fsolve(get_bound, Bound)
        Bounds[t] = Bound[0]
  
        t -= 1
         
    return Bounds

np.random.seed(1234)

U_shocks = np.random.rand(J,T)
prod_shocks = np.random.randn(J,T)
jj_shocks = np.random.rand(J,T)
mu_draws = np.random.rand(J,T)
p_draws1 = np.random.randn(J,T)
p_draws2 = np.random.randn(J,T)
p_draws3 = np.random.randn(J,T)
p_draws4 = np.random.randn(J,T)
lamb_shocks = np.random.rand(J,T)  


def simulate(params):
    p0, delta, alpha, s, lamb = params
    sigma = get_sigma(params)
    Bounds = get_bounds(params,sigma)
  
    emp = np.zeros((J,T))
    exp = np.zeros((J,T))
    p = np.zeros((J,T))
    wage = np.zeros((J,T))
    job_num = np.zeros((J,T))
    job_length = np.zeros((J,T))
    tenure = np.zeros((J,T))
    find = np.zeros((J,T))
    separate = np.zeros((J,T))
    jj = np.zeros((J,T))
    new_job = np.zeros((J,T))
    mu = np.zeros((J,T))
  
    for j in xrange(J):
        for t in range(1,T):
            if emp[j,t-1]==1:
                # If worker employed last period, see if count goes up or down, 
                # see if still employed, update matrices
                U_shock = U_shocks[j,t]
                prod_shock = (sigma*prod_shocks[j,t]+mu[j,t-1])
                jj_shock = jj_shocks[j,t]
                if U_shock < s:
                    # Worker becomes unemployed
                    exp[j,t] = exp[j,t-1]
                    job_num[j,t] = job_num[j,t-1]
                    separate[j,t-1] = 1
  
                else:
                    # Worker remains employed, undergoes production shock
                    p[j,t] = f(prod_shock, p[j,t-1], 1, sigma)
                    # See if worker wants to quit
                    if p[j,t] > Bounds[int(exp[j,t-1])]:
                        # Stays employed
                        # See if gets outside offer  
                        if jj_shock > delta: 
                            # No outside offer
                            emp[j,t] = 1
                            exp[j,t] = exp[j,t-1] + 1
                            job_num[j,t] = job_num[j,t-1]
                            if job_num[j,t]==1:
                                job_length[j,t] = job_length[j,t-1] + 1
                            tenure[j,t] = tenure[j,t-1] + 1
                            wage[j,t] = w(p[j,t])
                            mu[j,t] = mu[j,t-1]
                        else:
                            # Outside offer
                            # gets draw from alpha*t job distribution
                            mu_draw = mu_draws[j,t]
                            if mu_draw < p0:
                                mu_temp = muh
                                std = sqrt(alpha*exp[j,t-1])*sigma
                                mean = alpha*exp[j,t-1]*muh                                    
                                p_draw = f((std*p_draws1[j,t]+mean), p0, mean/muh, sigma)
                            else:
                                mu_temp = mul
                                std = sqrt(alpha*exp[j,t-1])*sigma
                                mean = 0. #alpha*exp[j,t-1]*mul                                    
                                p_draw = f((std*p_draws2[j,t]+mean), p0, alpha*exp[j,t-1], sigma)
                            if p[j,t] > p_draw:
                                # Rejects offer
                                # Stays employed
                                emp[j,t] = 1
                                exp[j,t] = exp[j,t-1] + 1
                                job_num[j,t] = job_num[j,t-1]
                                if job_num[j,t]==1:
                                    job_length[j,t] = job_length[j,t-1] + 1
                                tenure[j,t] = tenure[j,t-1] + 1
                                wage[j,t] = w(p[j,t])
                                mu[j,t] = mu[j,t-1]
                            else:  
                                # Accepts offer
                                # Starts new job
                                emp[j,t] = 1
                                p[j,t] = p_draw
                                exp[j,t] = exp[j,t-1] + 1
                                job_num[j,t] = job_num[j,t-1]+1
                                new_job[j,t] = 1
                                tenure[j,t] = 1
                                separate[j,t-1] = 1
                                jj[j,t-1] = 1
                                mu[j,t] = mu_temp
                                wage[j,t] = w(p[j,t])
                    else:
                        # Quits job
                        exp[j,t] = exp[j,t-1]
                        job_num[j,t] = job_num[j,t-1]
                        separate[j,t-1] = 1
            else: 
                # If unemployed last period worker searches to find new job
                lamb_shock = lamb_shocks[j,t]
                if lamb_shock < lamb:
                    # Worker gets draw from alpha*t job distribution
                    mu_draw = mu_draws[j,t]
                    if exp[j,t-1]==0:
                        p[j,t] = p0
                    else:
                        if mu_draw < p0:
                            std = sqrt(alpha*exp[j,t-1])*sigma
                            mean = alpha*exp[j,t-1]*muh
                            p[j,t] = f((std*p_draws3[j,t]+mean), p0, mean/muh, sigma)
                        else:
                            std = sqrt(alpha*exp[j,t-1])*sigma
                            mean = 0. #alpha*exp[j,t-1]*mul
                            p[j,t] = f((std*p_draws4[j,t]+mean), p0, alpha*exp[j,t-1], sigma)
                    # Check to see if worker wants to accept job
                    if p[j,t] > Bounds[int(exp[j,t-1])]:
                        # Accepts job
                        emp[j,t] = 1
                        exp[j,t] = exp[j,t-1] + 1
                        find[j,t-1] = 1
                        new_job[j,t] = 1
                        job_num[j,t] = job_num[j,t-1] + 1
                        if job_num[j,t]==1:
                            job_length[j,t] = 1
                        tenure[j,t] = tenure[j,t-1] + 1
                        wage[j,t] = w(p[j,t])
                        if mu_draw < p0:
                            mu[j,t] = muh
                        else:
                            mu[j,t] = mul
                    else:
                        # Reject job
                        exp[j,t] = exp[j,t-1]
                        job_num[j,t] = job_num[j,t-1]
  
                else:
                    # Worker doesn't get job offer. Remains unemployed
                    exp[j,t] = exp[j,t-1]
                    job_num[j,t] = job_num[j,t-1]
        # if j%1000==0:
            # print j
    return emp, find, separate, jj, job_length
 
def get_simul_moments(params):
    emp, find, separate, jj, job_length = simulate(params)
    
    # Calculate various moments from simulated data.
    find_rate = np.sum(find,0)/(J-np.sum(emp,0))
  
    ave_find_rate = np.mean(find_rate.reshape(T/12,12),1)
      
    # Mean duration of first jobs to be 13.7 months (if they last less than 10 years). 
    first_job_dur = (job_length>0).sum(1)
    mask = first_job_dur > 120
    first_job_dur[mask] = 0
    ave_first_job_dur = first_job_dur.sum()/float(J-mask.sum())
  
    # And for 58% of separations to be to new jobs. 
    ave_j2j_10 = np.sum(jj[:156])/np.sum(separate[:156])
     
    # Finally, target 5 year age averages of job finding probabilities over the life cycle starting 
    # with 21-25 year old workers as a set of targets for alpha.
    ave_find_rate_simul = ave_find_rate[3:-2].reshape(7,5).mean(1)
      
    # adding targets for job separation probabilities of the 51-55 year old group
    # and adding a target to the job finding probabilities for 18-21 year olds. 
    ave_sep_rate_simul = (separate[:,-7:-2].sum() - jj[:,-7:-2].sum())/(emp[:,-7:-2].sum())
    ave_find_rate_simul_18_21 = ave_find_rate[:3].mean()
    
    moments = np.concatenate(([ave_first_job_dur],[ave_j2j_10],
                              ave_find_rate_simul,[ave_sep_rate_simul],
                              [ave_find_rate_simul_18_21]))
    return moments
 
find_sep_data = np.loadtxt('./FindSepAge.csv', delimiter=',', skiprows=1)
ave_find_rate_data = find_sep_data[5:40,4].reshape(7,5).mean(1)
ave_find_rate_data_18_21 = find_sep_data[5:40,4].reshape(7,5).mean(1)
ave_sep_rate_data = find_sep_data[35:40,5].mean()
ave_find_rate_data_18_21 = find_sep_data[2:5,4].mean()
data_moments = np.concatenate(([13.7], [.58], ave_find_rate_data,
                               [ave_sep_rate_data],[ave_find_rate_data_18_21]))
 
def MSE(params):
    simul_moments = get_simul_moments(params)
    err = (simul_moments-data_moments)/data_moments
    objective = err.T.dot(err)
    print params, objective, simul_moments 
    return objective
     
import time
 
t0=time.time()
best_params = fmin(MSE,(0.65, 0.2, 0.10, 0.015, 0.4),xtol=0.001,ftol=0.001)
t1=time.time()

print (t1-t0)/60

#[ 0.64692542  0.22536716  0.09768947  0.01414697  0.39198156]
#[ 0.64665979  0.22562383  0.09762958  0.01410207  0.39243804]

