
#Import Wage Data
D <- read.table("C:/ExpWorkFlowSimData/D0_wage.csv",,header=FALSE,sep=",")

#rename variables
names(D)<-paste(rep("X.",40),1:40,sep="")
#assign id
D$id<-1:length(D[,1])

S<-reshape(D,
#identify individuals
idvar=c("id"),
#identify everything with numbers in the name as something that is varying
varying=names(D[,1:40]),
direction="long")



################

#Import Employment Data
D <- read.table("C:/ExpWorkFlowSimData/D0_emp.csv",,header=FALSE,sep=",")

#rename variables
names(D)<-paste(rep("X.",40),1:40,sep="")
#assign id
D$id<-1:length(D[,1])

s<-reshape(D,
#identify individuals
idvar=c("id"),
#identify everything with numbers in the name as something that is varying
varying=names(D[,1:40]),
direction="long")


S<-cbind(S,s[,3])


################

#Import Experience Data
D <- read.table("C:/ExpWorkFlowSimData/D0_exp.csv",,header=FALSE,sep=",")

#rename variables
names(D)<-paste(rep("X.",40),1:40,sep="")
#assign id
D$id<-1:length(D[,1])

s<-reshape(D,
#identify individuals
idvar=c("id"),
#identify everything with numbers in the name as something that is varying
varying=names(D[,1:40]),
direction="long")


S<-cbind(S,s[,3])

################

#Import Job Number Data
D <- read.table("C:/ExpWorkFlowSimData/D0_jobnum.csv",,header=FALSE,sep=",")

#rename variables
names(D)<-paste(rep("X.",40),1:40,sep="")
#assign id
D$id<-1:length(D[,1])

s<-reshape(D,
#identify individuals
idvar=c("id"),
#identify everything with numbers in the name as something that is varying
varying=names(D[,1:40]),
direction="long")


S<-cbind(S,s[,3])

################

#Import Tenure Data
D <- read.table("C:/ExpWorkFlowSimData/D0_tenure.csv",,header=FALSE,sep=",")

#rename variables
names(D)<-paste(rep("X.",40),1:40,sep="")
#assign id
D$id<-1:length(D[,1])

s<-reshape(D,
#identify individuals
idvar=c("id"),
#identify everything with numbers in the name as something that is varying
varying=names(D[,1:40]),
direction="long")

S<-cbind(S,s[,3])


#############

#Rename variables

names(S)<-c("id","time","wage","emp", "exp", "jobnum", "tenure")




####################

#Begin Analysis

#Order Variables
S<-S[order(S$id,S$time),]

#Subset Time-- do not want to use all 40 years since only have approx 27 years of NLSY
S<-S[S$time<20,]

#Cut out dates where unemployed
S<-S[S$wage!=0,]

#Did you get a new job this year?
library(Hmisc)
S$lastjobnum<-ave(S$jobnum,S$id,FUN=function(x)Lag(x,shift=1))
S$deltajob<-S$jobnum-S$lastjobnum
S$newjob<-(is.na(S$deltajob) | S$deltajob>0)*1


#Observable Job Number
S$jobno<-ave(S$newjob,S$id,FUN=cumsum)


#Job Id
S$jobid <- as.numeric(factor(interaction(S$id, S$jobno))) 
S$jobexp<-ave(S$exp,S$jobid,FUN=min)

#Tenure in job
S$since<-ave(S$time,S$jobid,FUN=function(x)Lag(x,shift=1))
S$since<-S$time-S$since
S[is.na(S$since),]$since<-0
S$tenure2<-ave(S$since,S$jobid,FUN=cumsum)

#Create log wage
S$logwage<-log(S$wage)

# Create the log difference in wages off of the initial wage in each job for each year of tenure.
S$protodelta<-(S$tenure2==0)*S$wage
S$initwage<-ave(S$protodelta,S$jobid,FUN=max)
S$lagwage<-log(S$wage)-log(S$initwage)

#head(S)
summary(S)

# export data frame to Stata binary format 
library(foreign)
write.dta(S, "C:/ExpWorkFlowSimData/D0_to_STATA.dta")

